package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.SysMenuMapper;
import com.huiju.welcome.mapper.SysRoleMapper;
import com.huiju.welcome.mapper.SysRoleMenuMapper;
import com.huiju.welcome.model.SysMenu;
import com.huiju.welcome.model.SysRole;
import com.huiju.welcome.model.SysRoleMenu;
import com.huiju.welcome.service.ISysRoleService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 角色业务
 * @author weiximei
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements ISysRoleService {

    @Autowired
    private SysMenuMapper sysMenuMapper;

    @Autowired
    private SysRoleMenuMapper sysRoleMenuMapper;

    @Override
    public ResponseBean getList(String roleName, Integer pageNum, Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();

        IPage<SysRole> page = new Page<>();
        page.setCurrent(pageNum);
        page.setSize(pageSize);

        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(roleName), "role_name", roleName);
        IPage<SysRole> sysRoleIPage = this.page(page, queryWrapper);
        responseBean.addSuccess(sysRoleIPage);
        return responseBean;
    }

    @Override
    public ResponseBean getByRoleId(Integer roleId) {
        ResponseBean responseBean = new ResponseBean();

        SysRole role = this.getById(roleId);
        if (null ==  role) {
            responseBean.addError("角色不存在！");
            return responseBean;
        }

        // 查询出该角色下所有菜单
        List<SysMenu> sysMenus = sysMenuMapper.selectRoleMenuList(role.getRoleId());
        role.setMenuList(sysMenus);
        responseBean.addSuccess(role);
        return responseBean;
    }

    @Override
    public ResponseBean roleSaveOrUpdate(SysRole role, String [] menuIdArray) {
        ResponseBean responseBean = new ResponseBean();

        // 存在roleId表示修改，否则添加
        if (null == role.getRoleId()) {
            role.setCreateDate(LocalDateTime.now());
            role.setStatus(1);
        }

        boolean result = this.saveOrUpdate(role);
        if (!result) {
            responseBean.addError("角色修改错误！");
            return responseBean;
        }

        // 删除菜单，并重新添加


        QueryWrapper<SysRoleMenu> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("role_id", role.getRoleId());
        sysRoleMenuMapper.delete(queryWrapper);

        List<SysRoleMenu> sysRoleMenuList = Arrays.asList(menuIdArray).stream()
                .map(e -> new SysRoleMenu(role.getRoleId(), e + ""))
                .collect(Collectors.toList());
        sysRoleMenuMapper.insertBeach(sysRoleMenuList);

        responseBean.addSuccess("操作成功！");
        return responseBean;
    }

    @Override
    public ResponseBean getMenus() {
        ResponseBean responseBean = new ResponseBean();

        QueryWrapper<SysMenu> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("menu_pid","-1");
        queryWrapper.eq("status","1");
        List<SysMenu> sysMenus = sysMenuMapper.selectList(queryWrapper);
        getMenuChildren(sysMenus);
        responseBean.addSuccess(sysMenus);
        return responseBean;
    }

    /**
     * 查询子节点
     * @param sysMenus
     */
    private void getMenuChildren(List<SysMenu> sysMenus) {
        sysMenus.forEach(e -> {
            QueryWrapper<SysMenu> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("menu_pid",e.getMenuId());
            queryWrapper.eq("status","1");
            List<SysMenu> sysMenuChildren = sysMenuMapper.selectList(queryWrapper);
            if (null != sysMenuChildren) {
                getMenuChildren(sysMenuChildren);
            }
            e.setChildren(sysMenuChildren);
        });
    }
}
